//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//

/* jshint sub:true */
/* globals Game: true */

Game.DynamicGlyph = function(properties) {
    properties = properties || {};
    Game.Glyph.call(this,properties);

    this._name = properties['name'] || '';
    this._aname = properties['aname'] || 'a '+this._name;
    this._serialNumber = Game.DynamicGlyph.prototype.nextSerialNumber++;

    this._attachedMixins = {};
    this._attachedMixinInterfaces = {};
    this._listeners = {};
    
    var mixins = properties['mixins'] || [];
    for(var i=0; i<mixins.length; i++) {
        for(var key in mixins[i]) {
            if (key !== 'init' && key !== 'name' && !this.hasOwnProperty(key)) {
                this[key] = mixins[i][key];
            }
        }

        this._attachedMixins[mixins[i].name] = true;

        if (mixins[i].interfaceName) {
            this._attachedMixinInterfaces[mixins[i].interfaceName] = true;
        }

        if (mixins[i].listeners) {
            for (key in mixins[i].listeners) {
                if (mixins[i].listeners.hasOwnProperty(key)) {
                    if (!this._listeners[key]) {
                        this._listeners[key] = [];
                    }
                    this._listeners[key].push(mixins[i].listeners[key]);
                }
            }
        }
        
        if (mixins[i].init) {
            mixins[i].init.call(this, properties);
        }
    }
};

Game.DynamicGlyph.extend(Game.Glyph);

Game.DynamicGlyph.prototype.nextSerialNumber = 1;

Game.DynamicGlyph.prototype.getSerialNumber = function() {
    return this._serialNumber;
};

Game.DynamicGlyph.prototype.hasMixin = function(obj) {
    if (typeof obj === 'object') {
        return this._attachedMixins[obj.name];
    } else {
        return this._attachedMixins[obj] || this._attachedMixinInterfaces[obj];
    }
};

Game.DynamicGlyph.prototype.setName = function(name) {
    this._name = name;
};

Game.DynamicGlyph.prototype.getName = function() {
    return this._name;
};


Game.DynamicGlyph.prototype.getAName = function() {
    return this._aname;
};

Game.DynamicGlyph.prototype.raiseEvent = function(event) {
    if (this._listeners[event]) {
        var args = Array.prototype.slice.call(arguments, 1);
        var results = [];
        for(var i=0; i < this._listeners[event].length; i++) {
            results.push(this._listeners[event][i].apply(this, args));
        }
        return results;
    }
    return [];
};