//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//

/* globals Game: true */
/* jshint bitwise: false */

Game.Map.Farm = function(player) {
    var width = Game.getScreenWidth();
    var height = Game.getScreenHeight();
    Game.Map.call(this, this._generateMapTiles(width, height, player), { isOutside: true });
    
    player.setX(1);
    player.setY(height/2);
    player.setZ(0);
    this.addEntity(player);
};
Game.Map.Farm.extend(Game.Map);

Game.Map.Farm.prototype._fillCircle = function(tiles, centreX, centreY, radius, tile) {
    var x = radius;
    var y = 0;
    var xChange = 1 - (radius << 1);
    var yChange = 0;
    var radiusError = 0;
    
    while(x >= y) {
        for(var i = centreX-x; i <= centreX+x; i++) {
            tiles[i][centreY-y] = tile;
            tiles[i][centreY+y] = tile;
        }
        for(i = centreX-y; i <= centreY+y; i++) {
            tiles[i][centreX-x] = tile;
            tiles[i][centreX+x] = tile;
        }
        
        y++;
        radiusError += yChange;
        yChange += 2;
        if (((radiusError << 1) + xChange) > 0) {
            x--;
            radiusError += xChange;
            xChange += 2;
        }
    }
};

Game.Map.Farm.prototype._fillRect = function(tiles,leftX,topY,rightX,bottomY,tile) {
    for(var i = leftX; i <= rightX; i++) {
        for(var j = topY; j < bottomY; j++) {
            tiles[i][j] = tile;
        }
    }
};

Game.Map.prototype._getRandomGrassPosition = function(width,height,tiles) {
    var x,y;
    do {
        x = Math.floor(Math.random() * width);
        y = Math.floor(Math.random() * height);
    } while(tiles[x][y] !== Game.Tile.grassTile);
    return {x: x, y: y};
};



Game.Map.Farm.prototype._generateMapTiles = function(width, height, player) {
    var tiles = new Array(width);
    var x,y;
    for(x=0; x<width-1; x++) {
        tiles[x] = new Array(height);
        for(y=0; y<height; y++) {
            tiles[x][y] = Game.Tile.grassTile;
        }
    }
    tiles[width-1] = new Array(width);
    for(y=0; y<height; y++) {
        tiles[width-1][y] = Game.Tile.hillTile;
    }
    tiles[width-1][height/2] = Game.Tile.caveEntranceTile;
    
    for(var i=0; i<3; i++) {
        var leftX = Math.floor(Math.random()*(width-15))+5;
        var topY = Math.floor(Math.random()*(height-7)+3);
        var rightX = leftX + Math.floor(Math.random()*8)+2;
        var bottomY = topY  + Math.floor(Math.random()*5)+2;
        this._fillRect(tiles,leftX,topY,rightX,bottomY,Game.Tile.fieldTile);
    }
    
    this._fillCircle(tiles,Math.floor(Math.random()*(width-25))+10,Math.floor(Math.random()*(height-18))+7,Math.floor(Math.random()*5)+2,Game.Tile.waterTile);
    
    
    tiles[0][height/2] = Game.Tile.houseTile;
    
    var torianPos = this._getRandomGrassPosition(width,height,tiles);
    tiles[torianPos.x][torianPos.y] = Game.Tile.torianTile;
    
    return [tiles];
};