/*!
 * © 2014 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */

/* globals Game: true, ROT */
/* exported Game */

var Game = {
    _currentScreen: null,
    _screenWidth: 50,
    _screenHeight: 24,
    _display: null,
    _charSheetWidth: 30,
    _charSheetHeight: 24,
    _charSheet: null,
    _messagesWidth: 83,
    _messagesHeight: 10,
    _messages: null,
    _farm: null,
    _cave: null,
    _temple: null,

    init: function(mapdiv,chrdiv,msgdiv) {
        var canvas = document.createElement("canvas");
        var context = canvas.getContext("2d");
        context.font = "15px monospace";
        var charWidth = Math.ceil(context.measureText("W").width);
                
        this._screenWidth      = Math.floor(parseFloat(window.getComputedStyle(mapdiv).width)  / charWidth);
        this._charScreenWidth  = Math.floor(parseFloat(window.getComputedStyle(chrdiv).width)  / charWidth);
        this._messagesWidth    = Math.floor(parseFloat(window.getComputedStyle(msgdiv).width)  / charWidth);

        this._display = new ROT.Display({width: this._screenWidth, height: this._screenHeight});
        this._charSheet = new ROT.Display({width:this._charSheetWidth, height:this._charSheetHeight});
        this._messages = new ROT.Display({width:this._messagesWidth, height:this._messagesHeight});
        var game = this;
        var bindEventToScreen = function(event) {
            window.addEventListener(event, function(e) {
                // When an event is received, send it to the
                // screen if there is one
                if (game._currentScreen !== null) {
                    // Send the event type and data to the screen
                    game._currentScreen.handleInput(event, e);
                }
            });
        };
        // Bind keyboard input events
        bindEventToScreen('keydown');
        bindEventToScreen('keypress');
    },

    getDisplay: function() {
        return this._display;
    },
    
    getCharSheet: function() {
        return this._charSheet;
    },
    
    getMessages: function() {
        return this._messages;
    },
    
    getScreenWidth: function() {
        return this._screenWidth;
    },
    
    getScreenHeight: function() {
        return this._screenHeight;
    },
    
    switchScreen:function(screen) {
        if (this._currentScreen !== null) {
            this._currentScreen.exit();
        }
        this.getDisplay().clear();
        this._currentScreen = screen;
        if (this._currentScreen !== null) {
            this._currentScreen.enter();
        }
    },
    
    refresh: function() {
        this._display.clear();
        this._charSheet.clear();
        this._messages.clear();
        this._currentScreen.render(this._display,this._charSheet,this._messages);
    },
    setFarm: function(farm) {
        this._farm = farm;
    },
    getFarm: function() {
        return this._farm;
    },
    setCave: function(cave) {
        this._cave = cave;
    },
    getCave: function() {
        return this._cave;
    },
    setTemple: function(temple) {
        this._temple = temple;
    },
    getTemple: function() {
        return this._temple;
    }
};

window.onload = function() {
    // Check if rot.js can work on this browser
    if (!ROT.isSupported()) {
        var elt = document.createElement("p");
        elt.innerHTML = "The rot.js library isn't supported by your browser.";
        document.getElementById("roguelike").appendChild(elt);
        //alert("The rot.js library isn't supported by your browser.");
    } else {
        // Determine the basic elements
        var mapdiv = document.getElementById("roguelike34");
        var chrdiv = document.getElementById("charsheet");
        var msgdiv = document.getElementById("messages");
        // Initialize the game
        Game.init(mapdiv,chrdiv,msgdiv);
        // Add the container to our HTML page
        mapdiv.appendChild(Game.getDisplay().getContainer());
        chrdiv.appendChild(Game.getCharSheet().getContainer());
        msgdiv.appendChild(Game.getMessages().getContainer());
        // Load the start screen
        Game.switchScreen(Game.Screen.startScreen);
    }
};

Object.defineProperty(Object.prototype,"extend",{
    writable: true,
    enumerable: false,
    configurable: true,
    value: function(o) {
        var names = Object.getOwnPropertyNames(o);
        for (var i=0; i<names.length; i++) {
            if (names[i] in this)  { continue; }
            var desc = Object.getOwnPropertyDescriptor(o,names[i]);
            Object.defineProperty(this, names[i], desc);
        }
    }
});
