//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//

/* globals Game: true, ROT */

Game.Rules = {};

Game.Rules.d10 = function() {
    return Math.ceil(Math.random()*10);
};

Game.Rules.baseStat = function(character) {
    if (Game.Rules.formSelected("underlying") === "2014a") {
        return Game.Rules.baseStat2014a(character);
    } else {
        return Game.Rules.baseStat2011c(character);
    }
};



Game.Rules.baseStat2011c = function() {
    return (Game.Rules.d10()+Game.Rules.d10()+Game.Rules.d10())*3;
};

Game.Rules.baseStat2014a = function() {
    return Math.round((Game.Rules.d10()+Game.Rules.d10()+Game.Rules.d10())*2/3);
};

Game.Rules.newCharacter = function() {
    return { str:  Game.Rules.baseStat(),
             dex:  Game.Rules.baseStat(),
             agi:  Game.Rules.baseStat(),
             con:  Game.Rules.baseStat(),
             app:  Game.Rules.baseStat(),
             int:  Game.Rules.baseStat(),
             wit:  Game.Rules.baseStat(),
             will: Game.Rules.baseStat()
           };
};

Game.Rules.formSelected = function(field) {
    if (field === "underlying") { return Game.Rules.formUnderlyingSelected(); }
    var opts = document.forms.mods.elements[field];
    for(var i=0; i < opts.options.length; i++) {
        if (opts.options[i].selected) {
            if (opts.options[i].text === "Use Base Rule set") {
                return Game.Rules.formUnderlyingSelected();
            } else {
                return opts.options[i].text;
            }
        }
    }
    return "Unknown";
};

Game.Rules.formUnderlyingSelected = function() {
    var opts = document.forms.underlying.elements.underlying;
    for(var i=0; i < opts.options.length; i++) {
        if (opts.options[i].selected) {
                return opts.options[i].text;
        }
    }
    return "Unknown";
};

Game.Rules.startingMorale = function(character) {
    if (Game.Rules.formSelected("morale") === "Lower morale") {
        return Game.Rules.startingMoraleLower(character);
    } else if (Game.Rules.formSelected("morale") === "2014a") {
        return Game.Rules.startingMorale2014a(character);
    } else {
        return Game.Rules.startingMorale2011c(character);
    }
};

Game.Rules.startingMorale2011c = function(character) {
    return 100 + Math.ceil(character.con/2+character.will/2);
};

Game.Rules.startingMorale2014a = function(character) {
    return 100 + character.con*2+character.will;
};

Game.Rules.startingMoraleLower = function(character) {
    return 100 + Math.round((2*character.con+character.will)/5);
};

Game.Rules.woundPoints = function(character) {
    if (Game.Rules.formSelected("bodytype") === "2014a") {
        return Game.Rules.woundPoints2014a(character);
   } else {
       return Game.Rules.woundPoints2011c(character);
   }
};

/* jshint elision: true */
Game.Rules.woundPoints2011c = function(character) {
//                                  1    1    1    2    2    2    3    3    3    3    4    4    4    5    5    5    6    6    6    6    7    7    7    8    8    8    9    9    9    9
//                   3    6    9    2    5    8    1    4    7    0    3    6    9    2    5    8    1    4    7    0    3    6    9    2    5    8    1    4    7    0    3    6    9
    var leg   = [,,, 4,,, 4,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 7,,, 7,,, 7,,, 7,,, 7,,, 7,,, 8,,, 8,,, 8,,, 8,,, 8,,, 8,,, 9,,, 9,,, 9,,, 9,,, 9,,, 9];
    var abdo  = [,,, 3,,, 3,,, 3,,, 3,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6];
    var chest = [,,, 6,,, 7,,, 7,,, 7,,, 7,,, 8,,, 8,,, 8,,, 8,,, 9,,, 9,,, 9,,, 9,,,10,,,10,,,10,,,10,,,11,,,11,,,11,,,12,,,12,,,12,,,12,,,13,,,13,,,13,,,13,,,14,,,14,,,14,,,14,,,15];
    var arm   = [,,, 3,,, 3,,, 3,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 6,,, 7,,, 7,,, 7,,, 7,,, 7,,, 7,,, 7];
    var head  = [,,, 3,,, 3,,, 3,,, 3,,, 3,,, 3,,, 3,,, 3,,, 3,,, 3,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 4,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 5,,, 6,,, 6,,, 6,,, 6];
    
    return { leftLeg:   leg  [character.con],
             rightLeg:  leg  [character.con],
             abdomen:   abdo [character.con],
             chest:     chest[character.con],
             swordArm:  arm  [character.con],
             shieldArm: arm  [character.con],
             head:      head [character.con]
    };
};

Game.Rules.woundPoints2014a = function(character) {
//          0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20
var leg   = [, 7, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9,10,10,10,10,10,10];
var abdo  = [, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8];
var chest = [, 9, 9,10,10,10,11,11,11,11,11,12,12,12,13,13,13,13,14,14,15];
var arm   = [, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9];
var head  = [, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7];

var index = Math.ceil(character.con/5);

return { leftLeg:   leg  [index],
         rightLeg:  leg  [index],
         abdomen:   abdo [index],
         chest:     chest[index],
         swordArm:  arm  [index],
         shieldArm: arm  [index],
         head:      head [index]
       };
};

Game.Rules.damageScore = function(average) {
    if (Game.Rules.formSelected("damage") === "Normal Distribution") {
        return Game.Rules.normalDistDamage(average);
    } else if (Game.Rules.formSelected("damage") === "Exponential Distribution") {
        return Game.Rules.expDistDamage(average);
    } else if (Game.Rules.formSelected("damage") === "2014a") {
        return Game.Rules.normalDistDamage(average);
    } else {
        return Game.Rules.performance(average);
    }
};

Game.Rules.normalDistDamage = function(average) {
    var roll = ROT.RNG.getPercentage();
    var table = [];
    
    table[1] = [ 5, 50, 100];
    table[2] = [ 5, 21,  50, 90, 100];
    table[3] = [ 5, 14,  29, 50,  81, 96, 100];
    table[4] = [ 5, 11,  21, 34,  50, 74,  90, 97, 100];
    table[5] = [ 5,  9,  16, 26,  37, 50,  70, 85,  94, 98, 100];
    table[6] = [ 5,  9,  14, 21,  29, 39,  50, 67,  81, 90,  96, 98, 100];
    table[7] = [ 5,  8,  12, 17,  24, 32,  41, 50,  64, 77,  87, 93,  97, 99, 100];
    table[8] = [ 5,  7,  11, 15,  21, 27,  34, 42,  50, 63,  74, 83,  90, 95,  97, 99, 100];
    table[9] = [ 5,  7,  10, 14,  18, 23,  29, 36,  43, 50,  61, 72,  81, 88,  92, 96,  98, 99, 100];
    table[10]= [ 5,  7,   9, 12,  16, 21,  26, 31,  37, 43,  50, 60,  70, 78,  85, 90,  94, 97, 98, 99, 100];
    
    while(typeof average === "function") {
        average = average();
    }
    
    var row = table[average];
    var damage = 0;
    while(!row[damage] || roll > row[damage]) {
        damage++;
    }
    return damage;

};

Game.Rules.expDistDamage = function(average) {
    var roll = ROT.RNG.getPercentage();
    var table = [];
    
    table[1] = [ 0, 75,  94, 98, 100];
    table[2] = [ 0, 29,  50, 65,  75, 82, 87, 91, 94, 96, 97, 98, 98, 99, 99, 99, 100];
    table[3] = [ 0, 21,  37, 50,  60, 68, 75, 80, 84, 87, 90, 92, 94, 95, 96, 97,  98, 98, 98, 99, 99, 100];
    table[4] = [ 0, 16,  29, 41,  50, 58, 65, 70, 75, 79, 82, 85, 87, 89, 91, 93,  94, 95, 96, 96, 97, 100];
    table[5] = [ 0, 13,  24, 34,  43, 50, 56, 62, 67, 71, 75, 78, 81, 83, 86, 87,  89, 90, 92, 93, 94, 100];
    table[6] = [ 0, 11,  21, 29,  37, 44, 50, 55, 60, 65, 68, 72, 75, 78, 80, 82,  84, 86, 87, 89, 90, 100];
    table[7] = [ 0,  9,  18, 26,  33, 39, 45, 50, 55, 59, 63, 66, 70, 72, 75, 77,  80, 81, 83, 85, 86, 100];
    table[8] = [ 0,  8,  16, 23,  29, 35, 41, 45, 50, 54, 58, 61, 65, 68, 70, 73,  75, 77, 79, 81, 82, 100];
    table[9] = [ 0,  7,  14, 21,  27, 32, 37, 42, 46, 50, 54, 57, 60, 63, 66, 69,  71, 73, 75, 77, 79, 100];
    table[10]= [ 0,  7,  13, 19,  24, 29, 34, 39, 43, 46, 50, 53, 56, 59, 62, 65,  67, 69, 71, 73, 75, 100];
    
    var row = table[average];
    var damage = 0;
    while(!row[damage] || roll > row[damage]) {
        damage++;
    }
    return damage;

};

Game.Rules.normal = function(x,mean,sd) {
    var X = (x-mean)/sd;
    var T=1/(1+0.2316419*Math.abs(X));
    var D=0.3989423*Math.exp(-X*X/2);
    var Prob=D*T*(0.3193815+T*(-0.3565638+T*(1.781478+T*(-1.821256+T*1.330274))));
    if (X>0) {
        Prob=1-Prob;
    }
    return Math.round(Prob*100000)/100000;
};

Game.Rules.performance = function(skill,cap) {
    if (Game.Rules.formSelected("performance") === "2014a") {
        return Game.Rules.performance2014a(skill,cap);
    } else {
        return Game.Rules.performance2011c(skill,cap);
    }
};

Game.Rules.performance2011c = function(skill,cap) {
    var roll = ROT.RNG.getPercentage();
    // footnote: undefined -> Nan and hence roll > undefined is always false - so if only one parm is passed to performance, the (non-existent) cap is ignored :)
    if (roll > cap) {
        roll = cap;
    }
    var table = [];
//                0  0  0  0  0  0  0  0  0  0  1  1  1  1  1  1  1  1  1  1  2
//                0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7  8  9  0
    table[ 0] = [21,30,40,50,65,78,88,94,97,99,100];
    table[ 1] = [20,29,38,49,63,76,86,93,97,99,100];
    table[ 2] = [20,28,37,47,61,74,84,92,96,98,99,100];
    table[ 3] = [18,26,35,46,59,73,84,91,96,98,99,100];
    table[ 4] = [17,25,34,44,57,71,83,91,95,98,99,100];
    table[ 5] = [16,24,33,43,55,69,81,90,95,98,99,100];
    table[ 6] = [15,23,32,42,53,68,80,89,94,97,99,100];
    table[ 7] = [14,22,30,40,51,66,78,88,94,97,99,100];
    table[ 8] = [14,20,29,39,50,64,77,86,93,97,99,100];
    table[ 9] = [13,19,28,37,48,62,75,85,92,96,98,99,100];
    table[10] = [12,18,26,36,47,60,73,84,91,96,98,99,100];
    table[11] = [11,17,25,35,45,58,72,83,90,95,98,99,100];
    table[12] = [10,16,24,33,44,56,70,81,90,95,98,99,100];
    table[13] = [10,15,23,32,42,54,68,80,89,94,97,99,100];
    table[14] = [ 9,14,22,31,41,53,67,78,88,93,97,99,100];
    table[15] = [ 9,14,21,30,40,51,64,76,86,92,96,98,99,100];
    table[16] = [ 8,13,19,28,38,49,63,75,85,92,96,98,99,100];
    table[17] = [ 7,12,18,27,37,48,61,74,84,91,96,98,99,100];
    table[18] = [ 7,11,17,26,35,46,59,72,83,90,95,98,99,100];
    table[19] = [ 6,10,16,24,34,45,57,71,81,89,95,97,99,100];
    table[20] = [ 6,10,15,23,33,43,56,69,80,88,94,97,99,100];
    table[21] = [ 6,10,15,23,32,42,54,67,78,87,93,96,98,99,100];
    table[22] = [ 5, 8,14,21,30,40,52,66,77,86,93,96,98,99,100];
    table[23] = [ 4, 8,13,20,28,39,50,64,76,85,92,96,98,99,100];
    table[24] = [ 4, 7,12,19,27,37,49,62,74,84,91,95,98,99,100];
    table[25] = [ 4, 7,11,18,26,36,47,60,73,83,90,95,98,99,100];
    table[26] = [ 3, 6,10,16,25,35,46,58,71,82,89,94,97,99,100];
    table[27] = [ 3, 6,10,16,24,34,44,56,69,80,88,93,97,98,99,100];
    table[28] = [ 3, 6,10,15,23,32,43,55,67,78,87,93,96,98,99,100];
    table[29] = [ 2, 5, 8,14,21,30,41,53,66,78,86,92,96,98,99,100];
    table[30] = [ 2, 4, 8,13,20,29,40,51,65,76,85,92,96,98,99,100];
    table[31] = [ 2, 4, 7,12,19,28,38,50,63,75,84,91,95,98,99,100];
    table[32] = [ 2, 3, 6,11,18,26,37,48,61,73,83,90,95,98,99,100];
    table[33] = [ 2, 3, 6,10,17,25,35,47,59,72,82,89,94,97,99,100];
    table[34] = [ 2, 3, 6,10,16,24,34,45,57,69,80,88,93,96,98,99,100];
    table[35] = [ 1, 3, 5, 9,15,22,32,43,56,69,79,87,93,96,98,99,100];
    table[36] = [ 1, 2, 4, 8,14,21,31,42,54,67,78,86,92,96,98,99,100];
    table[37] = [ 1, 2, 4, 7,13,20,29,40,52,65,76,85,92,96,98,99,100];
    table[38] = [ 1, 2, 4, 7,12,19,28,39,51,64,75,84,91,95,98,99,100];
    table[39] = [ 1, 2, 3, 6,11,18,27,37,49,62,74,83,90,95,97,99,100];
    table[40] = [ 1, 2, 3, 6,11,17,26,36,48,60,72,81,88,94,97,98,99,100];
    table[41] = [ 1, 2, 3, 6,10,16,25,35,46,58,70,80,88,93,96,98,99,100];
    table[42] = [  , 1, 2, 5, 9,15,23,33,44,57,69,79,87,93,96,98,99,100];
    table[43] = [  , 1, 2, 4, 8,14,21,31,43,55,68,78,86,92,96,98,99,100];
    table[44] = [  , 1, 2, 4, 7,13,20,30,41,54,66,77,85,92,95,98,99,100];
    table[45] = [  , 1, 2, 4, 7,12,19,28,40,52,64,75,84,91,95,98,99,100];
    table[46] = [  , 1, 2, 4, 7,12,19,28,38,50,62,73,82,89,94,97,98,99,100];
    table[47] = [  , 1, 2, 3, 6,11,17,26,37,49,61,72,81,88,93,97,98,99,100];
    table[48] = [  , 1, 2, 3, 6,10,17,25,36,47,59,70,80,87,92,96,98,99,100];
    table[49] = [  ,  , 1, 2, 5, 9,15,23,33,45,58,70,80,87,92,96,98,99,100];
    table[50] = [  ,  , 1, 2, 4, 8,14,22,32,44,56,68,78,86,92,96,98,99,100];
    table[51] = [  ,  , 1, 2, 4, 7,13,20,30,42,55,67,77,85,91,95,98,99,100];
    table[52] = [  ,  , 1, 2, 4, 7,13,20,30,41,53,64,75,83,90,94,97,98,99,100];
    table[53] = [  ,  , 1, 2, 3, 7,12,19,29,39,51,63,74,83,89,94,97,98,99,100];
    table[54] = [  ,  , 1, 2, 3, 6,11,18,27,38,50,62,72,81,88,93,96,98,99,100];
    table[55] = [  ,  ,  , 1, 2, 5, 9,16,25,36,48,60,72,81,88,93,96,98,99,100];
    table[56] = [  ,  ,  , 1, 2, 5, 8,15,23,34,46,59,70,80,87,93,96,98,99,100];
    table[57] = [  ,  ,  , 1, 2, 4, 8,14,22,32,45,57,69,79,86,92,96,98,99,100];
    table[58] = [  ,  ,  , 1, 2, 4, 7,13,21,31,43,56,67,77,85,91,95,98,99,100];
    table[59] = [  ,  ,  , 1, 2, 4, 7,12,20,30,42,54,65,75,84,90,94,97,98,99,100];
    table[60] = [  ,  ,  , 1, 2, 3, 6,12,19,28,40,52,64,74,83,89,94,97,98,99,100];
    table[61] = [  ,  ,  ,  , 1, 3, 5,10,17,26,38,51,63,73,82,89,94,97,98,99,100];
    table[62] = [  ,  ,  ,  , 1, 2, 5, 9,16,25,36,49,61,72,81,88,93,96,98,99,100];
    table[63] = [  ,  ,  ,  , 1, 2, 4, 8,15,24,35,48,60,71,80,87,93,96,98,99,100];
    table[64] = [  ,  ,  ,  , 1, 2, 4, 8,14,22,33,46,58,69,79,86,92,96,98,99,100];
    table[65] = [  ,  ,  ,  , 1, 2, 4, 7,13,21,31,44,57,68,78,85,91,95,97,99,100];
    table[66] = [  ,  ,  ,  , 1, 2, 4, 7,12,20,31,43,55,66,76,84,90,94,97,98,100];
    table[67] = [  ,  ,  ,  ,  , 1, 3, 6,11,18,28,41,53,65,75,83,90,94,97,98,100];
    table[68] = [  ,  ,  ,  ,  , 1, 2, 5,10,17,27,39,52,63,74,82,89,94,97,98,100];
    table[69] = [  ,  ,  ,  ,  , 1, 2, 5, 9,16,25,37,50,62,72,81,88,93,96,98,100];
    table[70] = [  ,  ,  ,  ,  , 1, 2, 4, 8,15,24,35,49,60,71,80,87,92,96,98,100];
    table[71] = [  ,  ,  ,  ,  , 1, 2, 4, 8,14,22,34,47,59,70,79,86,92,95,98,100];
    table[72] = [  ,  ,  ,  ,  , 1, 2, 4, 7,13,22,33,45,57,68,77,85,90,94,97,100];
    table[73] = [  ,  ,  ,  ,  , 1, 2, 3, 7,12,20,31,44,56,66,76,84,90,94,97,100];
    table[74] = [  ,  ,  ,  ,  ,  , 1, 3, 6,11,18,29,42,54,65,75,84,90,94,97,100];
    table[75] = [  ,  ,  ,  ,  ,  , 1, 2, 5,10,17,27,40,53,64,74,82,89,93,96,100];
    table[76] = [  ,  ,  ,  ,  ,  , 1, 2, 5, 9,16,26,38,51,63,73,81,88,93,96,100];
    table[77] = [  ,  ,  ,  ,  ,  , 1, 2, 4, 8,15,24,36,50,61,72,80,87,92,96,100];
    table[78] = [  ,  ,  ,  ,  ,  , 1, 2, 4, 7,14,23,34,48,60,70,79,86,92,95,100];
    table[79] = [  ,  ,  ,  ,  ,  , 1, 2, 4, 7,13,22,33,46,58,68,77,85,90,94,100];
    table[80] = [  ,  ,  ,  ,  ,  ,  , 1, 3, 6,12,20,31,44,57,67,77,85,90,94,100];
    table[81] = [  ,  ,  ,  ,  ,  ,  , 1, 3, 5,11,19,29,43,55,66,76,84,90,94,100];
    table[82] = [  ,  ,  ,  ,  ,  ,  , 1, 2, 5,10,17,28,41,54,65,74,83,89,93,100];
    table[83] = [  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 9,16,26,39,52,63,73,82,88,93,100];
    table[84] = [  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 8,15,25,37,51,62,72,81,87,92,100];
    table[85] = [  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 8,14,24,36,49,60,70,79,86,91,100];
    table[86] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 7,12,22,33,47,59,69,78,86,91,100];
    table[87] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 6,11,20,32,46,58,68,77,85,90,100];
    table[88] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 5,10,19,30,44,56,67,76,84,90,100];
    table[89] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 5,10,17,28,42,55,65,75,83,89,100];
    table[90] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 9,16,27,40,53,64,74,82,88,100];
    table[91] = [  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 8,15,25,38,52,63,72,81,87,100];
    table[92] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 7,14,23,36,50,61,71,80,86,100];
    table[93] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 6,12,22,34,49,60,70,78,86,100];
    table[94] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 6,11,20,32,47,58,68,77,85,100];
    table[95] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 5,10,19,31,45,57,67,76,84,100];
    table[96] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 5, 9,17,29,43,56,66,75,83,100];
    table[97] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 9,16,27,41,54,65,74,82,100];
    table[98] = [  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 2, 4, 8,16,26,39,53,63,72,80,100];
    table[99] = [  ,  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 7,14,24,37,51,62,71,80,100];
    table[100]= [  ,  ,  ,  ,  ,  ,  ,  ,  ,  , 1, 3, 6,12,22,35,50,60,70,79,100];
    var row = table[skill];
    var perf = 0;
    while(!row[perf] || roll > row[perf]) {
        perf++;
    }
    return perf;
};

Game.Rules.performance2014a = function(skill,cap) {
    var roll = ROT.RNG.getPercentage();
    // footnote: undefined -> Nan and hence roll > undefined is always false - so if only one parm is passed to performance, the (non-existent) cap is ignored :)
    if (roll > cap) {
        roll = cap;
    }
    
    var mean = 0.65*skill+3;
    var sdlow  = 3.77-0.0585*skill;
    var sdhigh = 2.60+0.0585*skill;
    for(var perf=0; perf<20; perf++) {
        var need = Game.Rules.normal(perf,mean, (perf < mean ? sdlow : sdhigh));
        if (roll/100 < need) {
            return perf;
        }
    }
    return 20;
};

Game.Rules.adj = function(average) {
    if (Game.Rules.formSelected("adjectives") === "2014a") {
        return Game.Rules.adj2014a(average);
    } else {
        return Game.Rules.adj2011c(average);
    }
};

Game.Rules.aadj = function(average) {
    if (Game.Rules.formSelected("adjectives") === "2014a") {
        return Game.Rules.aadj2014a(average);
    } else {
        return Game.Rules.aadj2011c(average);
    }
};

Game.Rules.adj2011c = function(perf) {
    var adjs = [ "hopeless", "humiliating",  "ineffective", "inferior",   "inadequate", "mediocre",
                             "adequate",    "reasonable",  "sufficient", "satisfactory", "superior",
                             "excellent",   "admirable",   "impressive", "exceptional",  "superb",
                             "outstanding", "astonishing", "astounding", "amazing",      "awesome" ];
    if (perf < 0)  { perf = 0;  }
    if (perf > 20) { perf = 20; }
    return adjs[perf];
};

Game.Rules.aadj2011c = function(perf) {
    var adjs = [ "a hopeless", "a humiliating",  "an ineffective", "an inferior",   "an inadequate", "a mediocre",
                              "an adequate",    "a reasonable",  "a sufficient", "a satisfactory", "a superior",
                              "an excellent",   "an admirable",   "an impressive", "an exceptional",  "a superb",
                              "an outstanding", "an astonishing", "an astounding", "an amazing",      "an awesome" ];
    if (perf < 0)  { perf = 0;  }
    if (perf > 20) { perf = 20; }
    return adjs[perf];
};

Game.Rules.adj2014a = function(perf) {
    var adjs = [ "humiliating", "embarrassing", "useless",    "poor",        "inadequate", "mediocre",
                                "adequate",     "sufficient", "competant",   "proficient",  "capable",
                                "skillful",     "effective",  "admirable",   "excellent",   "impressive",
                                "superb",       "brilliant",  "astonishing", "amazing",     "awesome" ];
    if (perf < 0)  { perf = 0;  }
    if (perf > 20) { perf = 20; }
    return adjs[perf];
};

Game.Rules.aadj2014a = function(perf) {
    var adjs = [ "a humiliating", "an embarrassing", "a useless",    "a poor",         "an inadequate", "a mediocre",
                                  "an adequate",     "a sufficient", "a competant",    "a proficient",  "a capable",
                                  "a skillful",      "an effective", "an admirable",   "an excellent",  "an impressive",
                                  "a superb",        "a brilliant",  "an astonishing", "an amazing",    "an awesome" ];
    if (perf < 0)  { perf = 0;  }
    if (perf > 20) { perf = 20; }
    return adjs[perf];
};

Game.Rules.skills = {
    Knife: ['dex'],
    Sword: ['agi','str'],
    Club: ['str'],
    Brawl: ['str']
};

Game.Rules.cuberoot = function(n) {
    if (n === 0) { return 0; }
    if (n > 0) {
        return Math.pow(n,1/3);
    } else {
        return -Math.pow(-n,1/3);
    }
};

Game.Rules.skillround = function(skill) {
    return Math.round(skill*10-0.5)/10;
};

Game.Rules.skilllevel = function(name,sheet,dps) {
    var stat = 0;
    for(var i=0; i<Game.Rules.skills[name].length; i++) {
        if (sheet[Game.Rules.skills[name][i]] > stat) {
            stat = sheet[Game.Rules.skills[name][i]];
        }
    }
    if (Game.Rules.formSelected("underlying") === "2014a") {
        var base = Math.pow(6000,1+0.0992063206*(15-stat)/15);
        var guess = 10;
        var newguess = guess - (3*guess*guess*guess+38*guess*guess+40*guess-40000*dps/base) / (9*guess*guess + 76*guess + 40);
        while(Game.Rules.skillround(guess) !== Game.Rules.skillround(newguess)) {
            guess = newguess;
            newguess = guess - (3*guess*guess*guess+38*guess*guess+40*guess-40000*dps/base) / (9*guess*guess + 76*guess + 40);
        }
        return Game.Rules.skillround(guess);
    } else {
        var diff = 5;
        if (stat > 20) { diff = 4; }
        if (stat > 32) { diff = 3; }
        if (stat > 44) { diff = 2; }
        if (stat > 59) { diff = 1; }
        if (stat > 79) { diff = 0; }
        var need = [];
        need[0] = [0,1,2,4,5,8,10,15,20,25,30,35,40,50,60,70,80,90,100,110,120,135,150,165,180,195,215,235,255,275,295,320,340,365,390,420,445,475,505,535,570,605,640,675,710,750,790,830,875,920,
                   965,1000,1050,1100,1150,1200,1250,1300,1400,1450,1500,1550,1600,1700,1750,1800,1900,1950,2050,2100,2200,2250,2350,2400,2500,2600,2700,2850,2950,3050,3150,3250,3350,3450,3550,3650,
                   3750,3900,4000,4100,4250,4350,4450,4600,4700,4850,5000,5100,5250];
        need[1] = [0,1,2,4,6,10,15,20,25,30,35,40,50,60,70,80,90,100,110,125,140,155,170,190,205,225,245,270,290,315,340,365,395,420,450,480,515,545,580,620,655,695,735,775,820,865,910,955,1000,1050,
                   1100,1150,1200,1250,1350,1400,1450,1500,1600,1650,1700,1800,1850,1950,2000,2100,2150,2250,2350,2450,2500,2600,2700,2800,2900,3000,3100,3200,3300,3400,3500,3600,
                   3750,3850,3950,4100,4200,4350,4450,4600,4750,4850,5000,5150,5300,5450,5600,5750,5900,6050];
        need[2] = [0,1,3,5,7,10,15,20,25,30,40,45,55,65,75,90,100,110,125,140,160,175,195,215,235,255,280,305,330,355,385,415,445,475,510,545,580,620,660,700,740,785,830,880,925,975,1050,1100,1150,1200,
                   1250,1300,1400,1450,1500,1600,1650,1700,1800,1850,1950,2050,2100,2200,2300,2350,2450,2550,2650,2750,2850,2950,3050,3150,3250,3400,3500,3600,3700,3850,3950,4100,4200,4350,
                   4500,4650,4750,4900,5050,5200,5350,5500,5650,5800,6000,6150,6300,6500,6650,6850];
        need[3] = [0,1,3,5,8,10,15,20,30,35,40,50,65,70,85,100,110,125,140,160,175,195,215,240,260,285,310,340,370,400,430,460,495,530,570,610,650,690,735,780,825,875,925,980,1050,1100,1150,1200,1250,1350,
                   1400,1450,1550,1600,1700,1750,1850,1900,2000,2100,2150,2250,2350,2450,2550,2650,2750,2850,2950,3050,3150,3300,3400,3500,3650,3750,3900,4000,4150,4300,4450,
                   4550,4700,4850,5000,5150,5300,5500,5650,5800,5950,6150,6300,6500,6650,6850,7050,7250,7400,7600];
        need[4] = [0,1,3,5,10,15,20,25,30,40,50,60,70,80,95,105,120,140,155,175,195,215,240,265,290,315,345,375,405,440,475,510,550,585,630,670,715,765,810,860,915,965,1000,1100,1150,1200,1250,1350,1400,1450,
                   1550,1600,1700,1800,1850,1950,2050,2100,2200,2300,2400,2500,2600,2700,2800,2900,3050,3150,3250,3400,3500,3650,3750,3900,4000,4150,4300,4450,4600,4700,4900,
                   5050,5200,5350,5550,5700,5850,6050,6200,6400,6600,6800,6950,7150,7350,7550,7750,8000,8200,8400];
        need[5] = [0,2,3,5,10,15,20,25,35,40,50,65,75,85,100,115,135,150,170,190,215,235,260,290,315,345,375,410,445,480,520,560,600,645,690,735,785,835,890,940,1000,1050,1100,1200,1250,1300,1400,1450,1550,1600,
                   1700,1750,1850,1950,2050,2100,2200,2300,2400,2500,2600,2750,2850,3050,3200,3300,3450,3550,3700,3850,3950,4100,4250,4400,4550,4700,4850,5000,
                   5200,5350,5500,5700,5850,6050,6250,6400,6600,6800,7000,7200,7400,7650,7850,8050,8300,8500,8700,8750,8950,9200];
        
        var column = need[diff];
        var level = 0;
        while(level < 100 && dps > column[level]) { level++; }
        return level+1;
    }
};

Game.Rules.meleeLocation = function(locationMod) {
    if (Game.Rules.formSelected("bodytype") === "2014a") {
        return Game.Rules.meleeLocation2014a(locationMod);
    } else {
        return Game.Rules.meleeLocation2011c(locationMod);
    }
};

Game.Rules.meleeLocation2011c = function(locationMod) {
    var roll = ROT.RNG.getPercentage();
    if (locationMod === 0) {
        if (roll < 17) { return "rightLeg";  }
        if (roll < 33) { return "leftLeg";   }
        if (roll < 45) { return "abdomen";   }
        if (roll < 57) { return "swordArm";  }
        if (roll < 80) { return "chest";     }
        if (roll < 91) { return "shieldArm"; }
        return "head";
    } else if (locationMod === 1) {
        if (roll < 17) { return "rightLeg";  }
        if (roll < 57) { return "abdomen";   }
        if (roll < 80) { return "swordArm";  }
        return "head";
    } else {
        if (roll < 57) { return "abdomen";   }
        return "head";
    }
};

Game.Rules.meleeLocation2014a = function(locationMod) {
    var roll = ROT.RNG.getPercentage();
    if (locationMod === 0) {
        if (roll < 17) { return "rightLeg";  }
        if (roll < 33) { return "leftLeg";   }
        if (roll < 45) { return "abdomen";   }
        if (roll < 71) { return "chest";     }
        if (roll < 81) { return "swordArm";  }
        if (roll < 91) { return "shieldArm"; }
        return "head";
    } else if (locationMod === 1) {
        if (roll < 17) { return "rightLeg";  }
        if (roll < 71) { return "abdomen";   }
        if (roll < 81) { return "swordArm";  }
        return "head";
    } else {
        if (roll < 71) { return "abdomen";   }
        return "head";
    }
};

Game.Rules.woundFromGap = function(base,gap) {
    if (Game.Rules.formSelected("woundgap") === "2014a") {
        return Game.Rules.woundFromGap2014a(base,gap);
    } else {
        return Game.Rules.woundFromGap2011c(base,gap);
    }
};

Game.Rules.woundFromGap2011c = function(base,gap) {
    if (gap >= 10) {
        return 2*base;
    } else {
        return base;
    }
};

Game.Rules.woundFromGap2014a = function(base,gap) {
    if (gap >= 10) {
        return base+5;
    } else if (gap >=7) {
        return base+2;
    } else {
        return base;
    }
};
