//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//


/* globals Game: true, ROT */

Game.Screen = {};

Game.Screen.startScreen = {
    enter: function() { Game.refresh(); },
    exit: function() {  },
    render: function(display,charSheet,messages) {
        display.drawText(1,1,"%c{yellow}Spider Caves Roguelike");
        display.drawText(1,2,"Press [Enter] to start");
    },
    handleInput: function(inputType, inputData) {
        if (inputType === 'keydown') {
            if (inputData.keyCode === ROT.VK_RETURN) {
                Game.switchScreen(Game.Screen.playScreen);
            }
        }
    }
};

Game.Screen.playScreen = {
    _player: null,
    _gameEnded: false,
    _gameWon: false,
    _subScreen: null,
    
    setSubScreen: function(subScreen) {
/*        if (subScreen) {
            console.log('switching to '+subScreen._caption);
        } else {
            if (this._subScreen) {
                console.log('exiting '+this._subScreen._caption);
            }
        }*/
        this._subScreen = subScreen;
        Game.refresh();
    },
    
    enter: function() {
        this._player = new Game.Entity(Game.PlayerTemplate);
        var knife = new Game.Item(Game.KnifeTemplate);
        this.getPlayer().addItem(knife);
        this.getPlayer().wieldSword(knife);

        var map = new Game.Map.Farm(this.getPlayer());
        Game.setFarm(map);

        this.getPlayer().initialAct();
        
        map.getEngine().start(); // There will be a call to Game.refresh once the engine calls act on the player
    },
    
    move: function(dx,dy,dz) {
        var newX = this.getPlayer().getX() + dx;
        var newY = this.getPlayer().getY() + dy;
        var newZ = this.getPlayer().getZ() + dz;
        this.getPlayer().tryMove(newX, newY, newZ);
    },
    
    wait: function() {
        
    },
    
    exit: function() {  },

    render: function(display,charSheet,messages) {
        if (this._subScreen) {
            this._subScreen.render(display);
        } else {
            this.renderDisplay(display);
        }
        this.renderCharSheet(charSheet);
        this.renderMessages(messages);
    },
    
    renderDisplay: function(display) {
        var map = this.getPlayer().getMap();
        var screenWidth = Game.getScreenWidth();
        var screenHeight = Game.getScreenHeight();
        var topLeftX = Math.max(0, this.getPlayer().getX() - (screenWidth/2));
        topLeftX = Math.min(topLeftX, map.getWidth()-screenWidth);
        var topLeftY = Math.max(0, this.getPlayer().getY() - (screenHeight/2));
        topLeftY = Math.min(topLeftY, map.getHeight()-screenHeight);
        
        var visibleCells = {};
        var currentDepth = this.getPlayer().getZ();
        if (!map.isOutside()) {
            map.getFov(currentDepth).compute(this.getPlayer().getX(),
                       this.getPlayer().getY(),
                       this.getPlayer().getSightRadius(),
                       function(x,y,radius,visibility) {
                          visibleCells[x+','+y] = true;
                          map.setExplored(x, y, currentDepth);
                       }
                      );
        }
        
        for(var x=topLeftX; x<topLeftX+screenWidth; x++) {
            for (var y=topLeftY; y<topLeftY+screenHeight; y++) {
                if (map.isOutside() || map.isExplored(x,y,currentDepth)) {
                    var glyph = map.getTile(x,y,currentDepth);
                    if (map.isOutside() || visibleCells[x+','+y]) {
                        var items = map.getItemsAt(x, y, currentDepth);
                        if (items) {
                            glyph = items[items.length-1];
                        }
                        if (map.getEntityAt(x, y, currentDepth)) {
                            glyph = map.getEntityAt(x, y, currentDepth);
                        }
                    }

                    var foreground = ROT.Color.fromString(glyph.getForeground());
                    if (!map.isOutside() && !visibleCells[x+','+y]) {
                        foreground = ROT.Color.interpolate(foreground,[0,0,0],0.75);
                    }
                    var background = glyph.getBackground();
                    if (background === 'transparent') {
                        background = map.getTile(x,y,currentDepth).getBackground();
                        if (background === 'transparent') {
                            background = 'black';
                        }
                    }
                    display.draw(x-topLeftX,y-topLeftY,glyph.getChar(),ROT.Color.toHex(foreground),background);
                }
            }
        }
    },
    
    renderCharSheet: function(charSheet) {
        var player = this.getPlayer();
        var wounds = player.getWoundPoints();
        var armour = player.getArmour();
        charSheet.drawText( 7, 0,'┌──────────┐');
        charSheet.drawText( 7, 1,'│     /    │');
        charSheet.drawText(11,1, wounds.head.toString());
        charSheet.drawText(14,1, armour.head.toString());
        charSheet.drawText( 0, 2,'┌──────┴──────────┴──────┐');
        charSheet.drawText( 0, 3,'│    ┌─┐          ┌─┐    │');
        charSheet.drawText( 0, 4,'│  / │ │     /    │ │  / │');
        charSheet.drawText( 1, 4, wounds.swordArm.toString());
        charSheet.drawText( 4, 4, armour.swordArm.toString());
        charSheet.drawText(11, 4, wounds.chest.toString());
        charSheet.drawText(14, 4, armour.chest.toString());
        charSheet.drawText(21, 4, wounds.shieldArm.toString());
        charSheet.drawText(24, 4, armour.shieldArm.toString());
        charSheet.drawText( 0, 5,'│    │ │          │ │    │');
        charSheet.drawText( 0, 6,'└────┘ ├──────────┤ └────┘');
        charSheet.drawText( 7, 7,'│     /    │');
        charSheet.drawText( 0, 7, player.getSwordHand().getName());
        charSheet.drawText(20, 7, player.getShieldHand().getName());
        charSheet.drawText(11, 7, wounds.abdomen.toString());
        charSheet.drawText(14, 7, armour.abdomen.toString());
        charSheet.drawText( 7, 8,'├──────────┤');
        charSheet.drawText( 7, 9,'│    ┌┐    │');
        charSheet.drawText( 7,10,'│    ││    │');
        charSheet.drawText( 7,11,'│  / ││  / │');
        charSheet.drawText( 8,11, wounds.leftLeg.toString());
        charSheet.drawText(11,11, armour.leftLeg.toString());
        charSheet.drawText(14,11, wounds.rightLeg.toString());
        charSheet.drawText(17,11, armour.rightLeg.toString());
        charSheet.drawText( 7,12,'│    ││    │');
        charSheet.drawText( 7,13,'└────┘└────┘');
        
        charSheet.drawText(1,15,'Str:  '+("  "+player.sheet.str).slice(-2));
        charSheet.drawText(1,16,'Dex:  '+("  "+player.sheet.dex).slice(-2));
        charSheet.drawText(1,17,'Agi:  '+("  "+player.sheet.agi).slice(-2));
        charSheet.drawText(1,18,'Con:  '+("  "+player.sheet.con).slice(-2));
        charSheet.drawText(1,19,'App:  '+("  "+player.sheet.app).slice(-2));
        charSheet.drawText(1,20,'Int:  '+("  "+player.sheet.int).slice(-2));
        charSheet.drawText(1,21,'Wit:  '+("  "+player.sheet.wit).slice(-2));
        charSheet.drawText(1,22,'Will: '+("  "+player.sheet.will).slice(-2));
        
        charSheet.drawText(13,15,"Morale:   "+("   "+player.getMorale()).slice(-3));
        charSheet.drawText(13,16,"Wounds:   "+("   "+player.getTotalWounds()).slice(-3));
        
        var line=18;
        var skills = [];
        for(var key in player.dps) {
            if (player.dps.hasOwnProperty(key)) {
                var dps = player.dps[key];
                var level = Game.Rules.skilllevel(key,player.sheet,dps);
                skills.push({key:key, level:level});
            }
        }
        skills.sort(function(a,b) {
            return b.level - a.level;
        });
        for(var i=0; i<skills.length; i++) {
            charSheet.drawText(13,line,(skills[i].key+":       ").slice(0,8)+" "+("    "+skills[i].level).slice(-4));
            line++;
        }
    },
    
    renderMessages: function(mess) {
        var messages = this.getPlayer().getMessages();
        var line = 0;
        for(var i = 0; i < messages.length; i++) {
            line += mess.drawText(0,line,'%c{white}%b{black}'+messages[i]);
        }
    },
    
    handleInput: function(inputType, inputData) {
        if (this._gameEnded) {
            if (inputType === 'keydown' && inputData.keyCode === ROT.VK_RETURN) {
                if (this._gameWon) {
                    Game.switchScreen(Game.Screen.winScreen);
                } else {
                    Game.switchScreen(Game.Screen.loseScreen);
                }
            }
            return;
        }
        
        if (this._subScreen) {
            this._subScreen.handleInput(inputType, inputData);
            return;
        }
        if (inputType === 'keydown') {
            if (inputData.keyCode === ROT.VK_LEFT || inputData.keyCode === ROT.VK_NUMPAD4) {
                this.move(-1,0,0);
            } else if (inputData.keyCode === ROT.VK_RIGHT || inputData.keyCode === ROT.VK_NUMPAD6) {
                this.move(1,0,0);
            } else if (inputData.keyCode === ROT.VK_UP || inputData.keyCode === ROT.VK_NUMPAD8) {
                this.move(0, -1,0);
            } else if (inputData.keyCode === ROT.VK_DOWN || inputData.keyCode === ROT.VK_NUMPAD2) {
                this.move(0,1,0);
            } else if (inputData.keyCode === ROT.VK_NUMPAD7) {
                this.move(-1, -1,0);
            } else if (inputData.keyCode === ROT.VK_NUMPAD9) {
                this.move(1,-1,0);
            } else if (inputData.keyCode === ROT.VK_NUMPAD1) {
                this.move(-1,1,0);
            } else if (inputData.keyCode === ROT.VK_NUMPAD3) {
                this.move(1,1,0);
            } else if (inputData.keyCode === ROT.VK_NUMPAD5) {
                this.wait();
            } else if (inputData.keyCode === ROT.VK_I) {
                this.showItemSubScreen(Game.Screen.inventoryScreen, this.getPlayer().getItems(),"You are not carrying anything!");
            } else if (inputData.keyCode === ROT.VK_D) {
                this.showItemSubScreen(Game.Screen.dropScreen, this.getPlayer().getItems(),"You have nothing to drop!");
            } else if (inputData.keyCode === ROT.VK_U) {
                this.showItemSubScreen(Game.Screen.useScreen, this.getPlayer().getItems(),"You have nothing to use!");
            } else if (inputData.keyCode === ROT.VK_W) {
                if (inputData.shiftKey) {
                    this.showItemSubScreen(Game.Screen.wearScreen, this.getPlayer().getItems(),"You have nothing to wear!");
                } else {
                    this.showItemSubScreen(Game.Screen.wieldScreen, this.getPlayer().getItems(),"You have nothing to wield!");
                }
            } else if (inputData.keyCode === ROT.VK_S) {
                this.showItemSubScreen(Game.Screen.shieldScreen, this.getPlayer().getItems(),"You have nothing to use in your shield hand!");
            } else if (inputData.keyCode === ROT.VK_COMMA && !inputData.shiftKey) {
                var items = this.getPlayer().getMap().getItemsAt(this.getPlayer().getX(), this.getPlayer().getY(), this.getPlayer().getZ());
                if (!items) {
                    this.getPlayer().sendSelfMessage("There is nothing here to pick up!");
                } else if (items.length === 1) {
                    var item = items[0];
                    if (this.getPlayer().pickupItems([0])) {
                        this.getPlayer().sendSelfMessage("You pick up %s.",[item.getAName()]);
                    } else {
                        this.getPlayer().sendSelfMessage( "Your inventory is full! Nothing was picked up.");
                    }
                } else {
                    Game.Screen.pickupScreen.setup(this.getPlayer(), items);
                    this.setSubScreen(Game.Screen.pickupScreen);
                    return;
                }
            } else {
                return;
            }
        } else if (inputType === 'keypress') {
            var keyChar = String.fromCharCode(inputData.charCode);
            if (keyChar === '>') {
                this.move(0,0,1);
            } else if (keyChar === '<') {
                this.move(0,0,-1);
            } else {
                return;
            }
        }
        this.getPlayer().getMap().getEngine().unlock();
    },
    
    showItemSubScreen: function(subScreen, items, emptyMessage) {
        if (items && subScreen.setup(this.getPlayer(),items) > 0) {
            this.setSubScreen(subScreen);
        } else {
            this.getPlayer().sendSelfMessage(emptyMessage);
            Game.refresh();
        }
    },
    
    setGameEnded: function() {
        this._gameEnded = true;
    },
    
    setGameWon: function() {
        this._gameEnded = true;
        this._gameWon = true;
    },
    getPlayer: function() {
        return this._player;
    }
};

Game.Screen.winScreen = {
    enter: function() { Game.refresh(); },
    exit: function() {  },
    render: function(display,charSheet,messages) {
        for(var i = 0; i < 22; i++) {
            var r = Math.round(Math.random()*255);
            var g = Math.round(Math.random()*255);
            var b = Math.round(Math.random()*255);
            var background = ROT.Color.toRGB([r,g,b]);
            display.drawText(2, i+1, "%b{"+background+"}You win! Taking "+Game.Screen.playScreen._player.turns+" turns!");
        }
    },
    handleInput: function(inputType, inputData) {
    }
};

Game.Screen.loseScreen = {
        enter: function() { Game.refresh(); },
        exit: function() {  },
        render: function(display,charSheet,messages) {
            for(var i = 0; i < 20; i++) {
                display.drawText(2, i+1, "%b{red}You died after "+Game.Screen.playScreen._player.turns+" turns!");
            }
            display.drawText(2,22,"Reload the page to try again.");
        },
        handleInput: function(inputType, inputData) {
        }
    };

/* jshint sub:true */
Game.Screen.ItemListScreen = function(template) {
    this._caption = template['caption'];
    this._okFunction = template['ok'];
    this._canSelectItem = template['canSelect'];
    this._canSelectMultipleItems = template['canSelectMultipleItems'];
    this._hasNoItemOption = template['hasNoItemOption'];
    this._isAcceptableFunction = template['isAcceptable'] || function(x) { return x; };
};

Game.Screen.ItemListScreen.prototype.setup = function(player, items) {
    this._player = player;
    var count = 0;
    var that = this;
    this._items = items = items.map(function(item) {
        if (that._isAcceptableFunction(item)) {
            count++;
            return item;
        } else {
            return null;
        }
    });
    this._selectedIndicies = [];
    return count;
};

Game.Screen.ItemListScreen.prototype.getPlayer = function() {
    return this._player;
};

Game.Screen.ItemListScreen.prototype.render = function(display) {
    var letters = 'abcdefghijklmnopqrstuvwxyz';
    display.drawText(0,0,'%c{white}%b{black}'+this._caption);
    var row=2;
    if (this._hasNoItemOption) {
        display.drawText(0, row, '%c{white}%b{black}0) No item');
        row++;
    }
    for(var i=0; i<this._items.length; i++) {
        if (this._items[i]) {
            var letter = letters.substring(i,i+1);
            var suffix = '';
            if (this._items[i] === this.getPlayer().getArmour()) {
                suffix = " (wearing)";
            } else if (this._items[i] === this.getPlayer().getSwordHand()) {
                suffix = " (wielding in sword hand)";
            } else if (this._items[i] === this.getPlayer().getShieldHand()) {
                suffix = " (wielding in shield hand)";
            }
            var selected = (this._canSelectItem && this._canSelectMultipleItems && this._selectedIndicies[i])?'%c{black}%b{white}':'%c{white}%b{black}';
            display.drawText(0, row, selected+letter+') '+this._items[i].getAName()+suffix);
            row++;
        }
    }
};

Game.Screen.ItemListScreen.prototype.executeOkFunction = function() {
    var selectedItems = {};
    for (var key in this._selectedIndicies) {
        if (this._selectedIndicies.hasOwnProperty(key)) {
            if (!isNaN(parseInt(key,10))) {
                selectedItems[key] = this._selectedIndicies[key];
            }
        }
    }
    Game.Screen.playScreen.setSubScreen(null);
    if (this._okFunction(selectedItems)) {
        this.getPlayer().getMap().getEngine().unlock();
    }
};

Game.Screen.ItemListScreen.prototype.handleInput = function(inputType, inputData) {
    if (inputType === 'keydown') {
        if (  inputData.keyCode === ROT.VK_ESCAPE || 
              (  inputData.keyCode === ROT.VK_RETURN &&
                (!this._canSelectItem || Object.keys(this._selectedIndicies).length === 0)
              )
           ) {
            Game.Screen.playScreen.setSubScreen(null);
        } else if (inputData.keyCode === ROT.VK_RETURN) {
            this.executeOkFunction();
        } else if (this._canSelectItem && this._hasNoItemOption && (inputData.keyCode === ROT.VK_0 || inputData.keyCode === ROT.VK_NUMPAD_0)) {
            this._selectedIndicies = [];
            this.executeOkFunction();
        } else if (this._canSelectItem && inputData.keyCode >= ROT.VK_A && inputData.keyCode <= ROT.VK_Z) {
            var index = inputData.keyCode - ROT.VK_A;
            if (this._items[index]) {
                if (this._canSelectMultipleItems) {
                    if (this._selectedIndicies[index]) {
                        delete this._selectedIndicies[index];
                    } else {
                        this._selectedIndicies[index] = this._items[index];
                    }
                    Game.refresh();
                } else {
                    this._selectedIndicies = [];
                    this._selectedIndicies[index] = this._items[index];
                    this.executeOkFunction();
                }
            }
        }
    }
};

Game.Screen.inventoryScreen = new Game.Screen.ItemListScreen({
    caption: 'Inventory',
    canSelect: false
});

Game.Screen.pickupScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the items you wish to pickup',
    canSelect: true,
    canSelectMultipleItems: true,
    ok: function(selectedItems) {
        if (!this.getPlayer().pickupItems(Object.keys(selectedItems))) {
            this.getPlayer().sendSelfMessage( "Your inventory is full! Not all items were picked up.");
        }
        return true;
    }
});

Game.Screen.dropScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the item to drop',
    canSelect: true,
    canSelectMultipleItems: false,
    ok: function(selectedItems) {
        this.getPlayer().dropItem(Object.keys(selectedItems)[0]);
        return true;
    }
});

Game.Screen.wieldScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the item you wish to wield',
    canSelect: true,
    conSelectMultipleItems: false,
    hasNoItemOption: true,
    isAcceptable: function(item) {
        return item && item.hasMixin(Game.ItemMixins.Equippable) && item.isWieldable();
    },
    ok: function(selectedItems) {
        var keys = Object.keys(selectedItems);
        if (keys.length === 0) {
            this.getPlayer().unwieldSword();
            this.getPlayer().sendSelfMessage("You will now use your fist in combat.");
        } else {
            var item = selectedItems[keys[0]];
            this.getPlayer().unequip(item);
            this.getPlayer().wieldSword(item);
            this.getPlayer().sendSelfMessage("You are wielding "+item.getAName()+".");
        }
        Game.refresh();
    }
});

Game.Screen.shieldScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the item you wish to use as a shield',
    canSelect: true,
    conSelectMultipleItems: false,
    hasNoItemOption: true,
    isAcceptable: function(item) {
        return item && item.hasMixin(Game.ItemMixins.Equippable) && item.isShieldable();
    },
    ok: function(selectedItems) {
        var keys = Object.keys(selectedItems);
        if (keys.length === 0) {
            this.getPlayer().unwhieldShield();
            this.getPlayer().sendSelfMessage("You are using nothing in your shield hand.");
        } else {
            var item = selectedItems[keys[0]];
            this.getPlayer().unequip(item);
            this.getPlayer().wieldShield(item);
            this.getPlayer().sendSelfMessage("You are wielding "+item.getAName()+" in your shield hand.");
        }
        Game.refresh();
    }
});

Game.Screen.wearScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the item you wish to wear',
    canSelect: true,
    conSelectMultipleItems: false,
    hasNoItemOption: true,
    isAcceptable: function(item) {
        return item && item.hasMixin(Game.ItemMixins.Equippable) && item.isWearable();
    },
    ok: function(selectedItems) {
        var keys = Object.keys(selectedItems);
        if (keys.length === 0) {
            this.getPlayer().takeOff();
            this.getPlayer().sendSelfMessage("You are now not wearing armour.");
        } else {
            var item = selectedItems[keys[0]];
            this.getPlayer().unequip(item);
            this.getPlayer().wear(item);
            this.getPlayer().sendSelfMessage("You are wearing "+item.getAName()+".");
        }
        Game.refresh();
    }
});

Game.Screen.useScreen = new Game.Screen.ItemListScreen({
    caption: 'Choose the item you wish to use',
    canSelect: true,
    conSelectMultipleItems: false,
    hasNoItemOption: true,
    isAcceptable: function(item) {
        return item &&  item.hasMixin(Game.ItemMixins.Useable);
    },
    ok: function(selectedItems) {
        var keys = Object.keys(selectedItems);
        if (keys.length >= 0) {
            this.getPlayer().use(Object.keys(selectedItems)[0]);
        }
        Game.refresh();
    }
});