//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
/* jshint sub:true */

/* globals Game: true */

Game.Tile = function(properties) {
    properties = properties || {};
    Game.Glyph.call(this, properties);
    this._walkable = properties['isWalkable'] || false;
    this._diggable = properties['isDiggable'] || false;
    this._blocksLight = (properties['blocksLight'] !== undefined) ? properties['blocksLight'] : true;
    this._description = properties['description'] || null;
};

Game.Tile.extend(Game.Glyph);

Game.Tile.prototype.isWalkable = function() {
    return this._walkable;
};

Game.Tile.prototype.isDiggable = function() {
    return this._diggable;
};

Game.Tile.prototype.isBlockingLight = function() {
    return this._blocksLight;
};

Game.Tile.prototype.getDescription = function() {
    return this._description;
};

Game.getNeighbourPositions = function(x,y) {
    var tiles = [];
    for (var dX=-1; dX<2; dX++) {
        for(var dY=-1; dY<2; dY++) {
            if (dX !== 0 || dY !== 0) {
                tiles.push({x: x+dX, y: y+dY});
            }
        }
    }
    return tiles.randomize();
};

Game.Tile.nullTile = new Game.Tile({});
Game.Tile.floorTile = new Game.Tile({
    character: '∙',
    isWalkable: true,
    blocksLight: false
});
Game.Tile.wallTile = new Game.Tile({
    character: '█',
    foreground: 'goldenrod',
    isDiggable: true
});

Game.Tile.stairsUpTile = new Game.Tile({
    character: '<',
    foreground: 'white',
    isWalkable: true,
    blocksLight: false,
    description: "There are stairs leading up here."
});

Game.Tile.caveExitTile = new Game.Tile({
    character: '«',
    foreground: 'white',
    isWalkable: true,
    blocksLight: false,
    description: "You see daylight above."
});

Game.Tile.templeExitTile = new Game.Tile({
    character: '«',
    foreground: 'white',
    isWalkable: true,
    blocksLight: false,
    description: "There are stairs leading out of the temple here."
});

Game.Tile.templeEntranceTile = new Game.Tile({
    character: '»',
    foreground: 'white',
    isWalkable: true,
    blocksLight: false,
    description: "You see stairs leading down into some kind of ancient cairn."
});

Game.Tile.stairsDownTile = new Game.Tile({
    character: '>',
    foreground: 'white',
    isWalkable: true,
    blocksLight: false,
    description: "There are stairs leading down here."
});

Game.Tile.hillTile = new Game.Tile({
    character: '█',
    foreground: 'brown',
    isWalkable: false,
    blocksLight: true
});

Game.Tile.caveEntranceTile = new Game.Tile({
    character: '●',
    foreground: 'black',
    background: 'brown',
    isWalkable: 'true',
    blocksLight: 'true',
    description: "There is a mysterious Cave Entrance leading down... (note, when you descend, map generation will take a few seconds)"
});

Game.Tile.waterTile = new Game.Tile({
    character: '~',
    foreground: 'blue',
    background: 'black',
    isWalkable: false,
    blocksLight: false,
});

Game.Tile.houseTile = new Game.Tile({
    character: '⌂',
    foreground: 'black',
    background: 'green',
    isWalkable: true,
    blocksLight: false,
    description: "This is your home."
});

Game.Tile.torianTile = new Game.Tile({
    character: 'Ʌ',
    foreground: 'black',
    background: 'green',
    isWalkable: true,
    blocksLight: false,
});

Game.Tile.grassTile = new Game.Tile({
    character: '█',
    foreground: 'green',
    background: 'green',
    isWalkable: true,
    blocksLight: false,
});

Game.Tile.fieldTile = new Game.Tile({
    character: '▒',
    foreground: 'yellow',
    background: 'brown',
    isWalkable: true,
    blocksLight: false,
});

Game.Tile.altarTile = new Game.Tile({
    character: 'ᴨ',
    foreground: 'red',
    background: 'white',
    isWalkable: true,
    blocksLight: true,
    description: "There is an large immoveable rectangular stone laid here."
});