<?php
/*
Plugin Name: Spider Caves
Plugin URI: http://blog.tremlas.com/spider-caves-plugin-details/
Description: Adds a shortcode that inserts the spider caves game
Version: 3.76.0
Author: David Vines
Author URI: http://tremlas.com
License:  Creative Commons License
*/
/* © Copyright 2014, 2016 David Vines

   Creative Commons License

   This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
*/

// Debugging function in case we need it
if ( !function_exists('djv_write_log') ) {
	function djv_write_log ( $log )  {
		if ( true === WP_DEBUG ) {
			if ( is_array( $log ) || is_object( $log ) ) {
				error_log( print_r( $log, true ) );
			} else {
				error_log( $log );
			}
		}
	}
}

/**
 * Shortcode to add the version of the plugin
 */
function djv_spiderCaves_getSpiderCavesVersion_shortcode() {
	return 'Spider Caves Version 3.76.0';
}
add_shortcode( 'spidercavesVersion', 'djv_spiderCaves_getSpiderCavesVersion_shortcode' );

/**
 * Add the "Spider Caves" game. Only one per page is supported.
 */

// Now for a shortcode that can be used to get links and/or tables for the characters
function djv_spiderCaves_shortcode( $attr, $content = null ) {
	$plugin_url = plugin_dir_url(__FILE__);
	wp_enqueue_script('djv_spidercaves_toggle', $plugin_url.'js/toggle.js' );
	wp_enqueue_script('djv_spidercaves_rot', $plugin_url.'js/rot/rot.min.js' );
	wp_enqueue_script('djv_spidercaves_sprintf', $plugin_url.'js/sprintf/sprintf.min.js' );
	wp_enqueue_script('djv_spidercaves_game', $plugin_url.'js/spidercaves/game.js' );
	wp_enqueue_script('djv_spidercaves_screens', $plugin_url.'js/spidercaves/screens.js' );
	wp_enqueue_script('djv_spidercaves_glyph', $plugin_url.'js/spidercaves/glyph.js' );
	wp_enqueue_script('djv_spidercaves_dynamicglyph', $plugin_url.'js/spidercaves/dynamicglyph.js' );
	wp_enqueue_script('djv_spidercaves_tile', $plugin_url.'js/spidercaves/tile.js' );
	wp_enqueue_script('djv_spidercaves_builder', $plugin_url.'js/spidercaves/builder.js' );
	wp_enqueue_script('djv_spidercaves_map', $plugin_url.'js/spidercaves/map.js' );
	wp_enqueue_script('djv_spidercaves_cave', $plugin_url.'js/spidercaves/cave.js' );
	wp_enqueue_script('djv_spidercaves_farm', $plugin_url.'js/spidercaves/farm.js' );
	wp_enqueue_script('djv_spidercaves_temple', $plugin_url.'js/spidercaves/temple.js' );
	wp_enqueue_script('djv_spidercaves_item', $plugin_url.'js/spidercaves/item.js' );
	wp_enqueue_script('djv_spidercaves_itemmixins', $plugin_url.'js/spidercaves/itemmixins.js' );
	wp_enqueue_script('djv_spidercaves_items', $plugin_url.'js/spidercaves/items.js' );
	wp_enqueue_script('djv_spidercaves_rules', $plugin_url.'js/spidercaves/rules.js' );
	wp_enqueue_script('djv_spidercaves_entity', $plugin_url.'js/spidercaves/entity.js' );
	wp_enqueue_script('djv_spidercaves_entitymixins', $plugin_url.'js/spidercaves/entitymixins.js' );
	wp_enqueue_script('djv_spidercaves_entities', $plugin_url.'js/spidercaves/entities.js' );
	
	wp_enqueue_style('djv_spidercaves_spidercaves', $plugin_url.'css/spidercaves.css');
	
	$text  = '<p>Based on Wayland Games rule set 2014a, but you can modify the rules!</p>' . PHP_EOL;
	$text .= '<form name="underlying">' . PHP_EOL;
    $text .= '    <table>' . PHP_EOL;
	$text .= '        <tr>' . PHP_EOL;
	$text .= '        <td><p>Base rule set:</p></td>' . PHP_EOL;
	$text .= '        <td><select name="underlying"><option>2014a</option><option>2011c</option></select>' . PHP_EOL;
	$text .= '        <td><p><i>Note that this controls the scale used for character attributes' . PHP_EOL;
	$text .= '        and the table used for skill levels based on development points awarded.</i></p></td>' . PHP_EOL;
	$text .= '        </tr>' . PHP_EOL;
	$text .= '        </table>' . PHP_EOL;
	$text .= '        </form>' . PHP_EOL;
	$text .= '        <div onClick="toggle(\'MainTable\')" style="cursor: pointer;"><p><b>[+] More specific options and modifications</b></p></div>' . PHP_EOL;
	$text .= '        <div id="MainTable" style="display: none">' . PHP_EOL;
	$text .= '        <form name="mods">' . PHP_EOL;
	$text .= '        <table>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Body layout:</td>' . PHP_EOL;
	$text .= '        		<td><select name="bodytype"><option>Use Base Rule set</option><option>2014a</option><option>2011c</option></select></td>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Damage:</td>' . PHP_EOL;
	$text .= '        		<td><select name="damage"><option>Use Base Rule set</option><option>2014a</option><option>2011c</option><option>Normal Distribution</option><option>Exponential Distribution</option></select></td>' . PHP_EOL;
	$text .= '        		<td><p><i>Note that use of the named distributions are intended for experimentation with the rule sets!</i></p></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Melee location table:</td>' . PHP_EOL;
	$text .= '        		<td><select name="location"><option>Use Base Rule set</option><option>2014a</option><option>2011c</option></select>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Morale Points:</td>' . PHP_EOL;
	$text .= '        		<td><select name="morale"><option>Use Base Rule set</option><option>2014a</option><option>2011c</option><option>Lower morale</option></select></td>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Adjectives:</td>' . PHP_EOL;
	$text .= '        		<td><select name="adjectives"><option>Use Base Rule set</option><option>Option 2014a</option><option>2011c</option></select>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Extra wounds from major or devastating success:</td>' . PHP_EOL;
	$text .= '        		<td><select name="woundgap"><option>Use Base Rule set</option><option>Option 2014a</option><option>2011c</option></select>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '        		<tr>' . PHP_EOL;
	$text .= '        		<td>Performance:</td>' . PHP_EOL;
	$text .= '        		<td><select name="performance"><option>Use Base Rule set</option><option>Option 2014a</option><option>2011c</option></select>' . PHP_EOL;
	$text .= '        		<td></td>' . PHP_EOL;
	$text .= '        		</tr>' . PHP_EOL;
	$text .= '          </table>' . PHP_EOL;
	$text .= '          </form>' . PHP_EOL;
	$text .= '          </div>' . PHP_EOL;
	$text .= '          <div id="gamecontainer">' . PHP_EOL;
	$text .= '            <div id="roguelike34"></div>' . PHP_EOL;
	$text .= '            <div id="vsep">&nbsp;</div>' . PHP_EOL;
	$text .= '            <div id="charsheet"></div>' . PHP_EOL;
	$text .= '          </div>' . PHP_EOL;
	$text .= '          <div id="hsep">&nbsp;</div>' . PHP_EOL;
	$text .= '          <div id="messages"></div>' . PHP_EOL;
	
	return $text;
}

add_shortcode( 'spidercaves', 'djv_spiderCaves_shortcode' );

/* --------------------- Functions for updating plugins ------------------------------ */
function djv_spiderCaves_get_remote_update_from( $urlbase, $slug ) {
 $url = $urlbase . '/wp-content/uploads/jsons/' . $slug . '.json';
 $remote =  wp_remote_get( $url, array( 'timeout' => 10, 'headers' => array( 'Accept' => 'application/json' ) ) );
 if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && 200 == $remote['response']['code'] && ! empty( $remote['body'] ) ) {
  $remote['urlbase'] = $urlbase;
 }
 return $remote;
}
function djv_spiderCaves_get_remote_update_info( $slug ) {
 $remote = get_transient( 'djv_update_' . $slug );
 if (false == $remote) {
  if ( 'cubi' == gethostname() ) {
   $remote = djv_spiderCaves_get_remote_update_from( 'http://cubi.internal', $slug );
   if (false !== $remote) {
    set_transient( 'djv_update_' . $slug, $remote, 10 ); // Save for only 10 seconds here
   }
  }
 }
 if (false == $remote || is_wp_error($remote) || !isset( $remote['response']['code'] ) || 200 !== $remote['response']['code'] || empty( $remote['body'] ) ) {
  $remote = djv_spiderCaves_get_remote_update_from( 'https://blog.tremlas.com', $slug );
  set_transient( 'djv_update_' . $slug, $remote, 43200 ); // Save for 12 hours
 }
 if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && 200 == $remote['response']['code'] && ! empty( $remote['body'] ) ) {
  $body = json_decode( $remote['body'] );
  $info = new StdClass();
  $info->name = 'Spider Caves';
  $info->slug = $slug;
  $info->version = $body->version;
  $info->requires = $body->requires;
  $info->author = '<a href="' . $body->author_profile . '">' . $body->author . '</a>';
  $info->download_link = $remote['urlbase'] . '/' . $body->download_url;
  $info->trunk = $remote['urlbase'] . '/' . $body->download_url;
  $info->last_updated = $body->last_updated;
  $info->sections = array(
    'description'  => implode( "\n", $body->sections->description ),
    'installation' => implode( "\n", $body->sections->installation ),
    'changelog'    => implode( "\n", $body->sections->changelog ),
  );
  $info->homepage = $remote['urlbase'] . '/' . $body->homepage;
  return $info;
 }
 return false;
}

function djv_spiderCaves_check_for_update( $name, $slug ) {
 $info = djv_spiderCaves_get_remote_update_info( $slug );
 if ( false == $info ) {
  return false;
 }
 if ( version_compare( '3.76.0', $info->version, '<' ) && version_compare( $info->requires, get_bloginfo( 'version' ), '<' ) ) {
  $update = (object) array(
    'id'            => $name,
    'slug'          => $slug,
    'plugin'        => $name,
    'new_version'   => $info->version,
    'url'           => $info->homepage,
    'package'       => $info->download_link,
    'icons'         => array(),
    'banners'       => array(),
    'banners_rtl'   => array(),
    'tested'        => '',
    'requires_php'  => '',
    'compatability' => new StdClass()
  );
  return $update;
 } else {
  return false;
 }
}

function djv_spiderCaves_pre_set_site_transient_update_plugins( $transient ) {
 // Check for update
 $pluginphp = 'spiderCavesPlugin/spiderCavesPlugin.php';
 $pluginslug = 'spiderCavesPlugin';
 $update = djv_spiderCaves_check_for_update( $pluginphp, $pluginslug );
 if ( $update ) {
  $transient->response[$pluginphp] = $update;
 } else {
  // No update
  $item = (object) array(
    'id'            => $pluginphp,
    'slug'          => $pluginslug,
    'plugin'        => $pluginphp,
    'new_version'   => '3.76.0',
    'url'           => '',
    'package'       => '',
    'icons'         => array(),
    'banners'       => array(),
    'banners_rtl'   => array(),
    'tested'        => '',
    'requires_php'  => '',
    'compatability' => new StdClass()
  );
  $transient->no_update[$pluginphp] = $item;
 }
 return $transient;
}
add_filter( 'pre_set_site_transient_update_plugins', 'djv_spiderCaves_pre_set_site_transient_update_plugins' );

function djv_spiderCaves_plugin_info( $result, $action, $args ) {
 $slug = 'spiderCavesPlugin';
 if ( 'plugin_information' !== $action ) {
  return $result;
 }
 if ( $slug !== $args->slug) {
  return $result;
 }
 return djv_spiderCaves_get_remote_update_info( $slug );
}
add_filter( 'plugins_api', 'djv_spiderCaves_plugin_info', 10, 3 );

?>